import { SceneSBT } from "./scene.js";
import { ActorsSBT } from "./actors.js";
import { RegionsSBT } from "./regions.js";
import { ViewHtml } from "./view.js";

export class StepBeyondTime extends FormApplication {
    view;
    scene = new SceneSBT();
    actors = new ActorsSBT();
    regions = new RegionsSBT();
    static interval = {year: 0, month: 0, day: 0, hour: 0, minute: 0, seconds: 0};

    static get defaultOptions() {
        return foundry.utils.mergeObject(super.defaultOptions, {
            object: {},
            template: "./modules/step_beyond_time/templates/sbtView.hbs",
            popOut: true,
            resizable: true,
            title: game.i18n.localize("SBT.name"),
        });
    }

    async formLoading(html){
        this.view = new ViewHtml(html);
    }

    initScene(scene) {
        this.scene.setScene(scene);
        this.view.sceneToHtml(scene);
        this.actors.getActors(scene);
        this.actors.getActor();
        if (this.actors.mainActor){
            this.view.actorToHtml(this.actors.mainActor, scene);
        }
        this.actorsBootomRender(scene);
        //this.view.regionsToHtml(scene);
    }

    initActor(actor) {
        let scene = this.scene.getScene();
        this.actors.setActor(actor, scene);
        this.actors.setActors(scene);
        this.view.actorToHtml(actor, scene);
    }

    async changeActor() {
        let scene = this.scene.getScene();
        if (this.actors.mainActor){
            let interval = {};
            interval[`${scene._id}`] = this.view.getActorTimeFromForm();
            await this.actors.mainActor.setFlag('step_beyond_time', 'interval', interval);
        }
        this.actors.setActors(scene);
        this.actorsBootomRender(scene);
    }

    actorsBootomRender(scene){
        this.view.clear_actors();
        for (let actor_id of this.actors.actors_ids){
            let actor = game.actors.get(actor_id);
            this.view.setActorsHtml(actor, scene);
        }
    }

    choiseActor(actorId){
        this.actors.getActor(actorId);
        this.view.actorToHtml(this.actors.mainActor, this.scene.getScene());
    }

    deleteActor(actorId){
        let actor = game.actors.get(actorId);
        let scene = this.scene.getScene();
        if (actor){
            actor.unsetFlag('step_beyond_time', 'interval');
            this.actors.removeActor(actor._id, scene);
            this.actors.getActor();
            if (this.actors.mainActor){
                this.view.actorToHtml(this.actors.mainActor, scene);
            }
            this.actorsBootomRender(scene);
        }
    }

    changeRegionFormula(element){
        let scene = this.scene.getScene();
        let region = game.canvas.regions.get(element.id);
        scene.setFlag('step_beyond_time', `formula${element.id}`, element.value);
    }

    changeTime(actor, step) {
        let result = {};
        let interval = actor.getFlag('step_beyond_time', 'interval');
        for (const prop in interval[canvas.scene._id]) {
            result[prop] = interval[canvas.scene._id][prop] * step;
        }
        SimpleCalendar.api.changeDate(result);
    }

    setTimeOnWaypoint(waypoints){
        for (let i of waypoints){
            canvas.scene.regions.forEach(region=> {
                let resultregion = game.canvas.regions.get(region._id);
                console.log(resultregion.testPoint({x:i.x, y:i.y}))
                if (resultregion.testPoint({x:i.x, y:i.y})){
                    let formula = canvas.scene.getFlag('step_beyond_time', `formula${region._id}`);
                    if (formula){
                        let result = parseFormula(formula);
                        console.log("result", result)
                    }
                }
            });
        }
    }

    getData(app, html){
        const data = super.getData();
        data.footer = `${game.i18n.localize("MI.footer")}<a href="${game.i18n.localize("MI.boostSite")}">${game.i18n.localize("MI.boostSite")}</a>`;
        return data;
    }

    activateListeners(html) {
        this.formLoading(html);
        html.on('change', '.inputDateTime', (event) => {
            this.changeActor();
        });
        
        html.on('change', '.regionCost', (event) => {
            this.changeRegionFormula(event.target);
        });

        html.find('.dateActor').on('dragover', (event) => event.preventDefault());
        html.find('.dateActor').on('drop', async (event) => {
            const entityData = await JSON.parse(event.originalEvent.dataTransfer.getData("text/plain"));
            const actor = await fromUuid(entityData.uuid);
            this.initActor(actor);
        });

        html.find('.dateScene').on('dragover', (event) => event.preventDefault());
        html.find('.dateScene').on('drop', async (event) => {
            const entityData = await JSON.parse(event.originalEvent.dataTransfer.getData("text/plain"));
            let scene = await fromUuid(entityData.uuid);
            this.initScene(scene);
        });

        //actor on list
        html.on('click', '.actorImg', (event) => {
            this.choiseActor(event.target.id);
        });

        html.on('click', '.actorName', (event) => {
            this.choiseActor(event.target.id);
        });
        //
        //remove actor on list
        html.on('click', '.actorClose', (event) => {
            this.deleteActor(event.target.id);
        });
        

        html.on('click', '.buttonScene', (event) => {
            this.initScene(canvas.scene);
        });

        html.on('click', '.buttonActor', async (event) => {
            let token = canvas.tokens.controlled[0];
            if (token) {
                const actor = token.actor;
                this.initActor(actor);
            }
            else{
                ui.notifications.error(`Выберите токен или перенесите актёра из сайдбара.`);
            }
        });
    }
}

function parseFormula(formula) {
    // Функция принадлежит Laziriti, спасибо тебе^^
    // Регулярное выражение для поиска выражений вида xdy
    const diceRegex = /(\d+)d(\d+)/g;

    // Функция для замены выражений вида xdy на случайные числа
    function rollDice(match, numDice, dieSides) {
      let total = 0;
      for (let i = 0; i < parseInt(numDice); i++) {
        total += Math.floor(Math.random() * parseInt(dieSides)) + 1;
      }
      return total;
    }

    // Заменяем все выражения вида xdy на случайные числа
    const evaluatedFormula = formula.replace(diceRegex, rollDice);

    // Вычисляем итоговое значение выражения
    return eval(evaluatedFormula);
}

Hooks.on("updateToken", (token, updateData, options, userId) => {

    let waypoints = [];
    console.log("token.x",token.x);
    console.log("token.y",token.y);
    const dx = Math.abs(token.x - (updateData.x || token.x)) / canvas.grid.size;
    const dy = Math.abs(token.y - (updateData.y || token.y)) / canvas.grid.size;
    const hexDistance = Math.ceil(Math.max(dx, dy + (dx / 2)));
    console.log("hexDistance", hexDistance);
    let xPlus = 1;
    let yPlus = 1;
    if (updateData.x - token.x < 0){
        xPlus = -1;
    }
    if (updateData.y - token.y < 0){
        yPlus = -1;
    }
    for (let i = 0; i < hexDistance; i++) {
        waypoints.push({x: token.x + i * canvas.grid.size * xPlus, y: token.y + i * canvas.grid.size * yPlus})
    }
    //StepBeyondTime.singletone.setTimeOnWaypoint(waypoints);
    StepBeyondTime.singletone.changeTime(game.actors.get(token.actorId), hexDistance);
});

Hooks.on("tokenMove", (speaker,actor,token,character,scope,scene,region,behavior,event) => {
    console.log(speaker,actor,token,character,scope,scene,region,behavior,event);
    console.log("token", token);
    console.log("token", event);
});