export class ViewHtml{
    constructor(html){
        this.html = html;
        // Scene
        this.sceneContainer = html.find('.dateScene');
        this.sceneName = html.find('[id="labelScene"]');
        this.sceneId = html.find('[id="inputScene"]');
        // Actor
        this.actorName = html.find('[id="labelActor"]');
        this.actorId = html.find('[id="inputActor"]');
        this.actorImg = html.find('[id="dateActorPic"]');
        //Actor time
        this.actorSeconds = html.find('[id="inputSec"]');
        this.actorMinutes = html.find('[id="inputMin"]');
        this.actorHours = html.find('[id="inputHour"]');
        this.actorDays = html.find('[id="inputDay"]');
        this.actorMonths = html.find('[id="inputMonth"]');
        this.actorYears = html.find('[id="inputYear"]');
        //Actors
        this.actorsContainer = html.find('.dateActorsOnScene');
        //Regions
        this.regionsContainer = html.find('.dateRegions');
        this.regionFormula = html.find('.regionCost');
    }

    formatStringToCard(string) {
        let result = string.substr(0, 3);
        result.toLowerCase();
        return result;
    }

    sceneToHtml(scene){
        this.sceneName.text(scene.name);
        this.sceneId.val(scene._id);
        this.sceneContainer.css('background-image', `url(${scene.background.src})`);
        this.sceneContainer.css("background-size", "cover");
    }

    actorToHtml(actor, scene){
        //display actor on form
        this.actorName.text(actor.name);
        this.actorId.val(actor._id);
        this.actorImg.attr("src", actor.prototypeToken.texture.src);
        let interval = actor.getFlag('step_beyond_time', 'interval');
        if (!interval){
            interval = {};
            interval[scene._id] = {year: 0, month: 0, day: 0, hour: 0, minute: 0, seconds: 0};
        }
        let actorInterval = interval[scene._id];
        this.actorSeconds.val(actorInterval["seconds"]);
        this.actorMinutes.val(actorInterval["minute"]);
        this.actorHours.val(actorInterval["hour"]);
        this.actorDays.val(actorInterval["day"]);
        this.actorMonths.val(actorInterval["month"]);
        this.actorYears.val(actorInterval["year"]);
    }

    getActorTimeFromForm(){
        let actorInterval = {
            seconds: parseInt(this.actorSeconds.val()),
            minute: parseInt(this.actorMinutes.val()),
            hour: parseInt(this.actorHours.val()),
            day: parseInt(this.actorDays.val()),
            month: parseInt(this.actorMonths.val()),
            year: parseInt(this.actorYears.val())
        };
        return actorInterval;
    }

    clear_actors(){
        this.actorsContainer.empty();
    }

    setActorsHtml(actor, scene){
        let interval = actor.getFlag('step_beyond_time', 'interval');
        if (interval){
            let actorInterval = interval[scene._id];
            let actorCard = `
                <div class="actorCard">
                    <img class="actorImg" id="${actor._id}" src=${actor.prototypeToken.texture.src}/>
                    <label class="actorName" id="${actor._id}">${actor.name}</label>
                    <div class="actorDateTime">
                        <div class="actorClose actorInputDate">
                            <label class="closeActor" id="${actor._id}">X</label>
                        </div>
                        <div class="actorDateSec actorInputDate">
                            <label class="actorLabelDateTime">${this.formatStringToCard(game.i18n.localize("SBT.dateTime.sec"))}</label>
                            <label class="actorInputDateTime">${actorInterval["seconds"]}</label>
                        </div>
                        <div class="actorDateMin actorInputDate">
                            <label class="labelDateTime">${this.formatStringToCard(game.i18n.localize("SBT.dateTime.min"))}</label>
                            <label class="actorInputDateTime">${actorInterval["minute"]}</label>
                        </div>
                        <div class="actorDateHour actorInputDate">
                            <label class="labelDateTime">${this.formatStringToCard(game.i18n.localize("SBT.dateTime.hour"))}</label>
                            <label class="actorInputDateTime">${actorInterval["hour"]}</label>
                        </div>
                        <div class="actorDateDay actorInputDate">
                            <label class="labelDateTime">${this.formatStringToCard(game.i18n.localize("SBT.dateTime.day"))}</label>
                            <label class="actorInputDateTime">${actorInterval["day"]}</label>
                        </div>
                        <div class="actorDateMonth actorInputDate">
                            <label class="labelDateTime">${this.formatStringToCard(game.i18n.localize("SBT.dateTime.month"))}</label>
                            <label class="actorInputDateTime">${actorInterval["month"]}</label>
                        </div>
                        <div class="actorDateYear actorInputDate">
                            <label class="labelDateTime">${this.formatStringToCard(game.i18n.localize("SBT.dateTime.year"))}</label>
                            <label class="actorInputDateTime">${actorInterval["year"]}</label>
                        </div>
                    </div>
                </div>
                `;
            this.actorsContainer.append(actorCard);
        }
        
    }

    regionsToHtml(scene){
        this.regionsContainer.empty();
        for (let region of scene.regions){
            let regionCard = `
            <div class="regionCard">
                <div class="regionColor" style="background-color: ${region.color.css};"></div>
                <label class="regionName">${region.name}</label>
                <label class="regionId">${region._id}</label>
                <input class="regionCost" id="${region._id}"/>
            </div>
            `
            this.regionsContainer.append(regionCard);
            console.log(region);
        }
    }
}